//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// DownloadListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "DownloadListCtrl.h"
#include "otherfunctions.h" 
#include "updownclient.h"
#include "opcodes.h"
#include "ClientDetailDialog.h"
#include "FileDetailDialog.h"
#include "commentdialoglst.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MULE_ITEM_KEY (void*)0
#define MULE_FILE_KEY (void*)1

bool CMuleCtrlItem::IsAskedForAnotherFile()
{
	return (m_pSource ? (m_pSource->reqfile != m_pFile) : false); 
}

CMuleItemMgr::~CMuleItemMgr()
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);

	MuleRange Range = m_ListData.equal_range(MULE_ITEM_KEY);
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		CMuleCtrlItem* pItem = (CMuleCtrlItem*)g->second;
		delete pItem;
	}
}

CMuleCtrlItem* CMuleItemMgr::AddFile(CPartFile* pFile)
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);

	if(!pFile)
		return NULL;

	// Create item
	CMuleCtrlItem* pItem = new CMuleCtrlItem(pFile, NULL);
	// To be able to find item by CPartFile pointer
	m_ListData.insert(make_pair(pFile, pItem));
	// To be able to find file by item pointer
	m_ListData.insert(make_pair(pItem, pFile));

	// To be able to find all items
	m_ListData.insert(make_pair(MULE_ITEM_KEY, pItem));

	// To be able to find all files
	m_ListData.insert(make_pair(MULE_FILE_KEY, pFile));

	return pItem;
}

void CMuleItemMgr::RemoveFile(CMuleCtrlItem* pItem)
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);
	CSingleLock RemoveLock(&m_MutexRemove, TRUE);

	CPartFile* pFile = pItem->GetFile();
//	int nFound = 0;
	bool bFound = true;
	while(bFound)
	{
		bFound = false;
		for(MuleIter g = m_ListData.begin(); g != m_ListData.end(); g++)
		{
			if(g->second == pItem)
			{
				m_ListData.erase(g);
				bFound = true;
//				nFound++;
				break;
			}
		}
	}
//	if(nFound == 0)
//		TRACE("Didn't find source to delete when deleting file %g!\n", pFile);

	MuleRange Range = m_ListData.equal_range(MULE_ITEM_KEY);
//	bFound = false;
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second == pItem)
		{
			m_ListData.erase(g);
//			bFound = true;
			break;
		}
	}
//	if(!bFound)
//		TRACE("Didn't find item to delete when deleting file %g!\n", pFile);

	Range = m_ListData.equal_range(MULE_FILE_KEY);
//	bFound = false;
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second == pFile)
		{
			m_ListData.erase(g);
//			bFound = true;
			break;
		}
	}
//	if(!bFound)
//		TRACE("Didn't find file to delete when deleting file %g!\n", pFile);

	m_ListData.erase(pItem);
	m_ListData.erase(pFile);
	delete pItem;
}

CMuleCtrlItem* CMuleItemMgr::AddSource(CUpDownClient* pSource, CPartFile* pFile)
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);

	if(!pSource || !pFile)
		return NULL;

	CMuleCtrlItem* pItem = FindSource(pSource, pFile, FALSE);

	// Create item, if it's new
	if(!pItem)
	{
		pItem = new CMuleCtrlItem(pFile, pSource);
		// To be able to find item by CPartFile pointer
		m_ListData.insert(make_pair(pFile, pItem));
		// To be able to find item by CUpDownClient pointer
		m_ListData.insert(make_pair(pSource, pItem));
		// To be able to find all items
		m_ListData.insert(make_pair(MULE_ITEM_KEY, pItem));
	}

	return pItem;
}

void CMuleItemMgr::RemoveSource(CMuleCtrlItem* pItem)
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);
	CSingleLock RemoveLock(&m_MutexRemove, TRUE);

	CUpDownClient* pSource = pItem->GetSource();
	m_ListData.erase(pSource);
	MuleRange Range = m_ListData.equal_range(pItem->GetSource());
//	bool bFound = false;
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second == pItem)
		{
			m_ListData.erase(g);
//			bFound = true;
			break;
		}
	}
//	if(!bFound)
//		TRACE("Didn't find source %g to delete!\n", pSource);
	Range = m_ListData.equal_range(pItem->GetFile());
//	bFound = false;
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second == pItem)
		{
			m_ListData.erase(g);
//			bFound = true;
			break;
		}
	}
//	if(!bFound)
//		TRACE("Didn't find file %g to delete!\n", pSource);
	Range = m_ListData.equal_range(MULE_ITEM_KEY);
//	bFound = false;
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second == pItem)
		{
			m_ListData.erase(g);
//			bFound = true;
			break;
		}
	}
//	if(!bFound)
//		TRACE("Didn't find item %g to delete!\n", pItem);
	delete pItem;

}

void CMuleItemMgr::RemoveSource(CUpDownClient* pSource)
{
	// Removing all instances of source
//	int nFound = 0;
	bool bFound = true;
	while(bFound)
	{
		bFound = false;
		MuleRange Range = m_ListData.equal_range(pSource);
		for(MuleIter g = Range.first; g != Range.second; g++)
		{
			CMuleCtrlItem* pItem = (CMuleCtrlItem*)g->second;
			if(pItem->IsSource())
			{
				RemoveSource(pItem);
//				nFound++;
				bFound = true;
				break;
			}
		}
	}
//	if(nFound == 0)
//		TRACE("Didn't find ANY source %g to delete!\n", pSource);
}

CMuleCtrlItem* CMuleItemMgr::FindSource(CUpDownClient* pSource, CPartFile* pFile, BOOL bLock)
{
	CSingleLock AccessLock(&m_MutexAccess, bLock);

	// It's better to find by CUpDownClient key, since there are
	// much less files per source than sources per file
	MuleRange Range = m_ListData.equal_range(pSource);
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second)
		{
			CMuleCtrlItem* pItem = (CMuleCtrlItem*)g->second;
			if(pItem->GetFile() == pFile)
				return pItem;
		}
	}
	return NULL;
}

CMuleCtrlItem* CMuleItemMgr::FindFile(CPartFile* pFile)
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);

	MuleRange Range = m_ListData.equal_range(pFile);
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		if(g->second)
		{
			CMuleCtrlItem* pItem = (CMuleCtrlItem*)g->second;
			if(pItem->IsFile())
				return pItem;
		}
	}
	return NULL;
}

MuleRange CMuleItemMgr::GetAllFiles()
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);

	// We keep list of files by file key
	return m_ListData.equal_range(MULE_FILE_KEY);
}

MuleRange CMuleItemMgr::GetItemsByFile(CPartFile* pFile)
{
	CSingleLock AccessLock(&m_MutexAccess, TRUE);

	return m_ListData.equal_range(pFile);
}


// CDownloadListCtrl

#define DLC_DT_TEXT (DT_LEFT|DT_SINGLELINE|DT_VCENTER|DT_NOPREFIX|DT_END_ELLIPSIS)
#define DLC_BARUPDATE 512

//temp fix for dragging columns
#define GetColumnWidth(X)   (CListCtrl::GetColumnWidth(X))

// CDownloadListCtrl

// hmm some parts here might be overkill ;) ...
// oh well at least I can do/draw/paint everything I want in this window now ;)

IMPLEMENT_DYNAMIC(CDownloadListCtrl, CListBox)
CDownloadListCtrl::CDownloadListCtrl() 
{
	m_CurrentSortItem = 0;
	m_SortAscending = true;
	m_ShowSrc=false;  // Cax2
	m_lastSort=0;
	m_sortTime=0;
}

CDownloadListCtrl::~CDownloadListCtrl()
{
	EMULE_TRY

	if(m_PrioMenu.GetSafeHmenu())
		m_PrioMenu.DestroyMenu();
	if(m_FileMenu.GetSafeHmenu())
		m_FileMenu.DestroyMenu();

	EMULE_CATCH
}

void CDownloadListCtrl::Init()
{
	EMULE_TRY

	CImageList ilDummyImageList;
	ilDummyImageList.Create(1, 17,ILC_COLOR32 | ILC_MASK, 1, 1); 
	SetImageList(&ilDummyImageList, LVSIL_SMALL);
	SetStyle();
	SetColors();

	ModifyStyle(LVS_SINGLESEL,0);

	InsertColumn(0,GetResString(IDS_DL_FILENAME),LVCFMT_LEFT, 260);
	InsertColumn(1,GetResString(IDS_DL_SIZE),LVCFMT_RIGHT, 60);
	InsertColumn(2,GetResString(IDS_DL_TRANSF),LVCFMT_RIGHT, 65);
	InsertColumn(3,GetResString(IDS_DL_SPEED),LVCFMT_RIGHT, 65);
	InsertColumn(4,GetResString(IDS_DL_PROGRESS),LVCFMT_LEFT, 170);
	InsertColumn(5,GetResString(IDS_DL_SOURCES),LVCFMT_CENTER, 50);
	InsertColumn(6,GetResString(IDS_PRIORITY),LVCFMT_LEFT, 55);
	InsertColumn(7,GetResString(IDS_STATUS),LVCFMT_LEFT, 70);
	InsertColumn(8,GetResString(IDS_DL_REMAINS),LVCFMT_LEFT, 110);
	// Maverick
	InsertColumn(9,GetResString(IDS_DL_ULDL),LVCFMT_LEFT,60);
	InsertColumn(10,GetResString(IDS_QL_RATING),LVCFMT_LEFT,60);

	imagelist.Create(16,16,ILC_COLOR32|ILC_MASK,0,10);
	imagelist.SetBkColor(CLR_NONE);
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_NORMAL),16,16));			// 0
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_COMPROT),16,16));			// 1
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_MLDONKEY),16,16));			// 2
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_EDONKEYHYBRID),16,16));  	// 3
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_CREDIT_ONLY),16,16));      // 4
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_FRIEND_ONLY),16,16));      // 5
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_BANNED_ONLY),16,16));      // 6
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_26_ONLY),16,16));         	// 7
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS1),16,16));         	// 8
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS2),16,16));         	// 9
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS3),16,16));         	// 10
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS4),16,16));         	// 11
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS5),16,16));         	// 12
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTMINUS),16,16));	  	// 13 15
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTNONE),16,16));	  		// 14 16
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTPLUS),16,16));	  		// 15 17
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_NO),16,16));		// 16 18
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_EXCELLENT),16,16));	// 17 19
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_GOOD),16,16));		// 18 20
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_FAIR),16,16));	  	// 19 21
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_POOR),16,16));		// 20 22
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_FAKE),16,16));	  	// 21 23

	LoadImgLstIcon(NULL,0,0); //Destroy last icon

	CreateMenues();
	LoadSettings(CPreferences::tableDownload);

	//sortdlg Cax2
	if (theApp.glob_prefs->DoUseSort())
		SortInit(99);
	else {
		// Barry - Use preferred sort order from preferences
		int sortItem = theApp.glob_prefs->GetColumnSortItem(CPreferences::tableDownload);
		bool sortAscending = theApp.glob_prefs->GetColumnSortAscending(CPreferences::tableDownload);
		SetSortArrow(sortItem, sortAscending);
		SortItems(SortProc, sortItem + (sortAscending ? 0:10));
	}

	EMULE_CATCH
}

void CDownloadListCtrl::Localize() 
{
	EMULE_TRY

	if(GetSafeHwnd())
	{
	  CHeaderCtrl* pHeaderCtrl = GetHeaderCtrl();
	  HDITEM hdi;
	  hdi.mask = HDI_TEXT;
	  CString strRes;
  
	  strRes = GetResString(IDS_DL_FILENAME);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(0, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_DL_SIZE);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(1, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_DL_TRANSF);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(2, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_DL_SPEED);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(3, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_DL_PROGRESS);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(4, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_DL_SOURCES);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(5, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_PRIORITY);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(6, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_STATUS);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(7, &hdi);
	  strRes.ReleaseBuffer();
  
	  strRes = GetResString(IDS_DL_REMAINS);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(8, &hdi);
	  strRes.ReleaseBuffer();

	  // Maverick
	  strRes = GetResString(IDS_DL_ULDL);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(9, &hdi);
	  strRes.ReleaseBuffer();

	  strRes = GetResString(IDS_QL_RATING);
	  hdi.pszText = strRes.GetBuffer();
	  pHeaderCtrl->SetItem(10, &hdi);
	  strRes.ReleaseBuffer();
	}

	EMULE_CATCH
}

void CDownloadListCtrl::AddFile(CPartFile* toadd)
{
	EMULE_TRY

	CMuleCtrlItem* pItem = m_ItemMgr.AddFile(toadd);
    uint16 itemnr = GetItemCount();
    InsertItem(LVIF_PARAM,itemnr,0,0,0,0,(LPARAM)pItem);

	EMULE_CATCH
}

void CDownloadListCtrl::AddSource(CPartFile* owner,CUpDownClient* source,bool notavailable)
{
	EMULE_TRY
        
	CMuleCtrlItem* pItem = m_ItemMgr.AddSource(source, owner);

    if (!owner->srcarevisible)
            return;

    LVFINDINFO find;
    find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)m_ItemMgr.FindFile(owner);
    int nItem = FindItem(&find);
	if(nItem < 0) nItem = 0;
	while (GetItemCount() > nItem+1 && !((CMuleCtrlItem*)GetItemData(nItem+1))->IsFile())
		nItem++;
    InsertItem(LVIF_PARAM,nItem+1,0,0,0,0,(LPARAM)pItem);

	EMULE_CATCH
}


void CDownloadListCtrl::RemoveSource(CUpDownClient* source,CPartFile* owner)
{
	EMULE_TRY

	CMuleCtrlItem* pSource = NULL;
	if(owner)
		pSource = m_ItemMgr.FindSource(source, owner);

	if(pSource || !owner)
	{
		if(theApp.emuledlg->transferwnd.infolistctrl.GetType() == 1)
		{
			if(theApp.emuledlg->transferwnd.infolistctrl.GetClient() == source)
			{
				theApp.emuledlg->transferwnd.infolistctrl.SwitchToFile(NULL);
				theApp.emuledlg->transferwnd.UpdateInfoHeader();
			}
		}

		// Delete source only for this file
		if(owner)
		{
			CMuleCtrlItem* pParent = m_ItemMgr.FindFile(owner);

			LVFINDINFO find;
			find.flags = LVFI_PARAM;
			find.lParam = (LPARAM)pParent;
			int nItem = FindItem(&find);
			if(nItem < 0) nItem = 0;
			for(; nItem < GetItemCount(); nItem++)
			{
				CMuleCtrlItem* pItem = (CMuleCtrlItem*)GetItemData(nItem);
				if(pItem->GetSource() == source && pItem->GetFile() == owner)
				{
					DeleteItem(nItem);
					break;
				}
			}
			m_ItemMgr.RemoveSource(m_ItemMgr.FindSource(source, owner));
		}
		// Delete this source for all files
		else
		{
			for(uint16 nItem = 0; nItem < GetItemCount();)
			{
				CMuleCtrlItem* pItem = (CMuleCtrlItem*)GetItemData(nItem);
				if(pItem->GetSource() == source)
				{
					DeleteItem(nItem);
				}
				else
					nItem++;
			}
			m_ItemMgr.RemoveSource(source);
		}
	}
	EMULE_CATCH
}

void CDownloadListCtrl::RemoveFile(CPartFile* toremove)
{
	EMULE_TRY
	CMuleCtrlItem* pFile = m_ItemMgr.FindFile(toremove);
	if(pFile)
	{
		if(theApp.emuledlg->transferwnd.infolistctrl.GetType() == 2)
		{
			if(theApp.emuledlg->transferwnd.infolistctrl.GetFile() == toremove)
			{
				theApp.emuledlg->transferwnd.infolistctrl.SwitchToFile(NULL);
				theApp.emuledlg->transferwnd.UpdateInfoHeader();
			}
		}			
		LVFINDINFO find;
		find.flags = LVFI_PARAM;
		find.lParam = (LPARAM)pFile;
		int result = FindItem(&find);
		if(result != -1)
			DeleteItem(result);
	}
   	for(int i = 0; i < GetItemCount();)
	{
		CMuleCtrlItem* pItem = (CMuleCtrlItem*)GetItemData(i);
		if(pItem->IsSource() && pItem->GetFile() == toremove)
	   	{
		   	DeleteItem(i);
			m_ItemMgr.RemoveSource(pItem);
	   	}
		else
			i++;
   	}
	if(pFile)
		m_ItemMgr.RemoveFile(pFile);
	EMULE_CATCH
}

void CDownloadListCtrl::UpdateItem(void* toupdate)
{
	EMULE_TRY

	CMuleCtrlItem* pFile = m_ItemMgr.FindFile((CPartFile*)toupdate);
	if(pFile)
	{
		LVFINDINFO find;
		find.flags = LVFI_PARAM;
		find.lParam = (LPARAM)pFile;
		int result = FindItem(&find);
		if (result != -1)
		{
			pFile->m_dwUpdated = 0;
			Update(result);
			RedrawItems(result,result);
			theApp.sharedfiles->UpdateItem(((CKnownFile*)toupdate));
		}
	}
	else	// Then it's source
	{
		// Looking for all items with this source
		for(int i = 0; i < GetItemCount(); i++)
		{
			CMuleCtrlItem* pItem = (CMuleCtrlItem*)GetItemData(i);
			if(pItem->GetSource() == (CUpDownClient*)toupdate)
			{
				pItem->m_dwUpdated = 0;
				Update(i);
				RedrawItems(i, i);
			}
		}
	}

	EMULE_CATCH
}

#define WARN_PERIOD_OF_NO_PROGRESS 7

void CDownloadListCtrl::DrawFileItem(CDC *dc, int nColumn, LPRECT lpRect, CMuleCtrlItem *lpCtrlItem) 
{
	EMULE_TRY

	if(!lpCtrlItem->IsFile())
		return;

	if(lpRect->left < lpRect->right) 
	{

		CString buffer;
		CPartFile *lpPartFile = lpCtrlItem->GetFile();
		switch(nColumn)
		{
			case 0:		// file name
				{
					POINT ipoint = { lpRect->left, lpRect->top + 1 };
					int iIcon = lpPartFile->GetSourceCount() > 0 ? (lpPartFile->srcarevisible ? 13 : 15) : 14;
					imagelist.Draw(dc, iIcon, ipoint, ILD_NORMAL);
					lpRect->left += 18;
                    // now comments/ratings
			        if (lpPartFile->HasComment() || lpPartFile->HasRating())
			        {
				        int image=16;
				        if (lpPartFile->HasRating())
				        {
				        	switch(lpPartFile->GetRating())
				        	{
								case 0: 
									image=16; 
									break; 
								case 1: 
									image=21;
									break; 
								case 2: 
									image=20; 
									break; 
								case 3: 
									image=18; 
									break; 
								case 4: 
									image=19; 
									break; 
								case 5: 
									image=17; 
									break;
								default:
									image=16;
							}
						}
				        POINT point= {lpRect->left-4,lpRect->top+3};
				        imagelist.Draw(dc, image, point, ILD_NORMAL);
			        }
				    lpRect->left+=10;

                    // obaldin: warn if file had no progress
                    // for the last week
                    COLORREF crOldTxtColor;
                    bool warn=false;
                    if(lpPartFile->GetStatus()==PS_READY) {
                        CTimeSpan ts = CTime::GetCurrentTime() - lpPartFile->GetLastDownTransfer();
                        if(ts.GetTotalHours()>24*WARN_PERIOD_OF_NO_PROGRESS)
                            warn = true;
                    }
                    if(warn)
                        crOldTxtColor = dc->SetTextColor((COLORREF)RGB(190,60,60));
				    dc->DrawText(lpPartFile->GetFileName(), (int)strlen(lpPartFile->GetFileName()),lpRect, DLC_DT_TEXT);
                    if(warn)
                        dc->SetTextColor(crOldTxtColor);

				    lpRect->left-=9;
					lpRect->left -= 18;
					break;
				}

			case 1:		// size
				buffer = CastItoXBytes(lpPartFile->GetFileSize());
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
				break;

			case 2:		// transfered
				buffer = CastItoXBytes(lpPartFile->GetTransfered());
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);	
				break;

			case 3:		// speed
            	//Cax2: faster if: this still shows 0.0 for all datarates between 0.01 and 0.05, nothing if lower...
            	if(lpPartFile->GetDatarate()<11)
					buffer = "";
				else
					buffer.Format("%.1f %s", lpPartFile->GetDatarate() / 1024.0f,GetResString(IDS_KBYTESEC));
	
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
				break;

			case 4:		// progress
				{
					lpRect->bottom --;
					lpRect->top ++;
					int  iWidth = lpRect->right - lpRect->left; 
					int iHeight = lpRect->bottom - lpRect->top;
					if (lpCtrlItem->m_Status == (HBITMAP)NULL)
						VERIFY(lpCtrlItem->m_Status.CreateBitmap(1, 1, 1, 8, NULL)); 
					CDC cdcStatus; 
					HGDIOBJ hOldBitmap;
					cdcStatus.CreateCompatibleDC(dc); 
					int cx = lpCtrlItem->m_Status.GetBitmapDimension().cx; 
					DWORD dwTicks = GetTickCount(); 
					if(lpCtrlItem->m_dwUpdated + DLC_BARUPDATE < dwTicks || cx !=  iWidth || !lpCtrlItem->m_dwUpdated)
					{ 
						lpCtrlItem->m_Status.DeleteObject(); 
						lpCtrlItem->m_Status.CreateCompatibleBitmap(dc,  iWidth, iHeight); 
						lpCtrlItem->m_Status.SetBitmapDimension(iWidth,  iHeight); 
						hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->m_Status); 

						RECT rec_status; 
						rec_status.left = 0; 
						rec_status.top = 0; 
						rec_status.bottom = iHeight; 
						rec_status.right = iWidth; 
						lpPartFile->DrawStatusBar(&cdcStatus,  &rec_status, theApp.glob_prefs->UseFlatBar()); 

						lpCtrlItem->m_dwUpdated = dwTicks + (rand() % 128); 
					} else 
						hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->m_Status); 

					dc->BitBlt(lpRect->left, lpRect->top, iWidth, iHeight,  &cdcStatus, 0, 0, SRCCOPY); 
					cdcStatus.SelectObject(hOldBitmap);
					lpRect->bottom ++;
					lpRect->top --;
				}
				break;
			case 5:		// sources
				{
					char buffer2[64];
					uint16 ncsc=lpPartFile->GetNotCurrentSourcesCount();
					if(ncsc>0)
						sprintf(buffer2,"%i/",lpPartFile->GetSourceCount() -ncsc);
					else
						sprintf(buffer2,"");
					buffer.Format("%s%i (%i)",buffer2, lpPartFile->GetSourceCount(), lpPartFile->GetTransferingSrcCount());
					if (((CKnownFile*)lpPartFile)->IsAutoPrioritized())
						((CKnownFile*)lpPartFile)->UpdateUploadAutoPriority();
					// theApp.sharedfiles->UpdateItem(((CKnownFile*)lpPartFile)); // obaldin: what the heck was it doing here? moved to UpdateItem
					dc->DrawText(buffer,(int)strlen(buffer),lpRect, DLC_DT_TEXT | DT_RIGHT);
				}
				break;

			case 6:		// prio
				// Tarod (auto priority)
				if (lpPartFile->IsAutoPrioritized())
				{
					lpPartFile->UpdateDownloadAutoPriority();
	
					switch(lpPartFile->GetPriority())
					{
						case PR_LOW:
							dc->DrawText(GetResString(IDS_PRIOAUTOLOW),GetResString(IDS_PRIOAUTOLOW).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_NORMAL:
							dc->DrawText(GetResString(IDS_PRIOAUTONORMAL),GetResString(IDS_PRIOAUTONORMAL).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_HIGH:
							dc->DrawText(GetResString(IDS_PRIOAUTOHIGH),GetResString(IDS_PRIOAUTOHIGH).GetLength(),lpRect, DLC_DT_TEXT);
							break;
					}
				} else {
					switch(lpPartFile->GetPriority())
					{
						case PR_LOW:
							dc->DrawText(GetResString(IDS_PRIOLOW),GetResString(IDS_PRIOLOW).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_NORMAL:
							dc->DrawText(GetResString(IDS_PRIONORMAL),GetResString(IDS_PRIONORMAL).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_HIGH:
							dc->DrawText(GetResString(IDS_PRIOHIGH),GetResString(IDS_PRIOHIGH).GetLength(),lpRect, DLC_DT_TEXT);
							break;
					}
				}
				break;

			case 7:		// status
				buffer.Format("%s", lpPartFile->GetPartfileStatus());
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				break;

			case 8:		// remaining time & size
				{

					// size 
					uint32 remains;
					remains=lpPartFile->GetFileSize()-lpPartFile->GetCompletedSize();

					if(remains < 0) 
						remains = 0;

					// time 
					sint32 restTime=lpPartFile->GetTimeRemaining();
					buffer = CastSecondsToHM(restTime) + " (" + CastItoXBytes(remains) + ")";
					if (lpPartFile->GetStatus()==PS_COMPLETING || lpPartFile->GetStatus()==PS_COMPLETE )
					buffer="";

					// jicxicmic
					float saved = 100.0f - lpPartFile->GetPercentCompleted(); 
					// [Lord KiRon] this is completely WRONG read the MSDN on format :)
					CString str1	=	buffer;
					//buffer.Format( "%s [%.1f%%]", buffer, saved);
					buffer.Format( "%s [%.1f%%]", str1, saved);
					// [//Lord KiRon]
					// jicxicmic - end
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				}
			    	break;
            // Maverick
			//case 9 & 10
			default:
					break;
		}
	}

	EMULE_CATCH
}

void CDownloadListCtrl::DrawSourceItem(CDC *dc, int nColumn, LPRECT lpRect, CMuleCtrlItem *lpCtrlItem) 
{
	EMULE_TRY

	if(!lpCtrlItem->IsSource())
		return;

	CString buffer;
	CUpDownClient *lpClient = lpCtrlItem->GetSource();

	CString sClientFileName = _T("");
	if(lpClient->GetClientFilename())
		try{ sClientFileName = CString(lpClient->GetClientFilename()); }catch(...){}
	CString sDownloadState = _T("");
	try{ sDownloadState = lpClient->GetDownloadStateAsString(); }catch(...){}
	uint8 nDownloadState = lpClient->GetDownloadState();
	eClientTypes nClientSoft = lpClient->GetClientSoft();
	uint32 nVersion;	//Cax2 - version nr bugfix
	switch(nClientSoft){
		case SO_EMULE:
		case SO_OLDEMULE:
		case SO_CDONKEY:
			nVersion= lpClient->GetMuleVersion();
			break;
		default:
			nVersion= lpClient->GetVersion();
	}
	bool bIsFriend = lpClient->IsFriend();
	float fRemoteBaseModifier = lpClient->GetRemoteBaseModifier();
	CString sClientName = _T("");
	if(lpClient->GetUserName())
		try{ sClientName = CString(lpClient->GetUserName()); }catch(...){}
	uint32 nTransferredDown = lpClient->GetTransferedDown();
	uint32 nDownloadDataRate = lpClient->GetDownloadDatarate();
	bool bIsEmuleClient = lpClient->IsEmuleClient();
	uint16 nRemoteQueueRank = lpClient->GetRemoteQueueRank();
	bool bRemoteQueueFull = lpClient->IsRemoteQueueFull();
	bool bIsRemoteInfoAvail = lpClient->IsRemoteInfoAvaiable();
	bool bCredits = lpClient->credits != (CClientCredits*)NULL && lpClient->credits != (CClientCredits*)-1;
	uint16 nRemoteRatio = lpClient->GetRemoteRatio();

	// Lord KiRon - just make compiler happy :)
 	POINT point;
 	CString status;

	switch(nColumn)
	{
		case 0:  	// icon, name
			// Lord Kiron
  			{
				RECT cur_rec;
				memcpy(&cur_rec, lpRect, sizeof(RECT));
				cur_rec.left += 18;
				point.x = cur_rec.left;
				point.y = cur_rec.top+1;
   				status = _T(""); 
				if(!lpCtrlItem->IsAskedForAnotherFile())
				{
					switch (nDownloadState)
					{
						case DS_CONNECTING:
						case DS_CONNECTED:
						case DS_WAITCALLBACK:
						case DS_TOOMANYCONNS:
							imagelist.Draw(dc, 10, point, ILD_NORMAL);
							break;
						case DS_ONQUEUE:
							imagelist.Draw(dc, 9, point, ILD_NORMAL);
							break;
						case DS_DOWNLOADING:
						case DS_REQHASHSET:
							imagelist.Draw(dc, 8, point, ILD_NORMAL);
							break;
						case DS_NONEEDEDPARTS:
						case DS_LOWTOLOWIP:
							imagelist.Draw(dc, 11, point, ILD_NORMAL);
							break;
						default:
							imagelist.Draw(dc, 12, point, ILD_NORMAL);
					}
				} else {
               		imagelist.Draw(dc, 11, point, ILD_NORMAL); 
					if(theApp.glob_prefs->IsA4AFStringEnabled()) {
						CString fname;
						fname.Format("(%s)", sClientFileName.GetBuffer(0));
						status = fname;
					} else
						status = _T("(") + GetResString(IDS_ASKED4ANOTHERFILE) + _T(")");
  				}
	
				cur_rec.left += 20;

                POINT point2= {cur_rec.left,cur_rec.top+1};

                if(nClientSoft == SO_EDONKEYHYBRID) {
                    imagelist.Draw(dc, 3, point2, ILD_NORMAL);
                    if (bIsFriend)
                        imagelist.Draw(dc, 5, point2, ILD_TRANSPARENT);
                    else if (fRemoteBaseModifier >= 1.1)
                        imagelist.Draw(dc, 4, point2, ILD_TRANSPARENT);
                }
		        else if (nClientSoft == SO_MLDONKEY) {
                    imagelist.Draw(dc, 2, point2, ILD_NORMAL);
                    if (bIsFriend)
                        imagelist.Draw(dc, 5, point2, ILD_TRANSPARENT);
                    else if (fRemoteBaseModifier >= 1.1)
                        imagelist.Draw(dc, 4, point2, ILD_TRANSPARENT);
		        }
		        else if (nClientSoft==SO_EMULE || nClientSoft==SO_OLDEMULE) {
                    imagelist.Draw(dc, 1, point2, ILD_NORMAL);
                    if (bIsFriend)
                        imagelist.Draw(dc, 5, point2, ILD_TRANSPARENT);
                    else if (fRemoteBaseModifier >= 1.1)
                        imagelist.Draw(dc, 4, point2, ILD_TRANSPARENT);
                }
				else {
                    imagelist.Draw(dc, 0, point2, ILD_NORMAL);
                    if (bIsFriend)
                        imagelist.Draw(dc, 5, point2, ILD_TRANSPARENT);
                    else if (fRemoteBaseModifier >= 1.1)
                        imagelist.Draw(dc, 4, point2, ILD_TRANSPARENT);
				}
				cur_rec.left += 20;

                //if(lpUpDownClient->GetClientSoft()==SO_EMULE && lpUpDownClient->GetMuleVersion()==0x26) {
                //	imagelist.Draw(dc, 7, point2, ILD_TRANSPARENT);
                //}
				
				CString sName;
				char* buffer1;
				if (sClientName.IsEmpty())
				{
					buffer1 = new char[50];
					sprintf(buffer1,"?  (%s)",GetResString(IDS_UNKNOWN));
				} else {
					buffer1 = new char[sClientName.GetLength() + 50 + status.GetLength()];
					sprintf(buffer1,"%s %s", sClientName.GetBuffer(0), status.GetBuffer(0));
				}

                // DonGato: different color for A4AF sources
				COLORREF crOldTxtColor;
				bool a4af=false;
				if(lpCtrlItem->IsAskedForAnotherFile())
					a4af = true;
				if(a4af)
					crOldTxtColor = dc->SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
				dc->DrawText(buffer1,(int)strlen(buffer1),&cur_rec, DLC_DT_TEXT);
                if(a4af)
                	dc->SetTextColor(crOldTxtColor);
				delete[] buffer1;
			}
			break;
			// Lord KiRon - end
		case 1:		// size
			break;

		case 2:		// transfered
			if(!lpCtrlItem->IsAskedForAnotherFile() && nTransferredDown)
			{
				buffer = CastItoXBytes(nTransferredDown);

				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);	
			}
			break;

		case 3:		// speed
			if (!lpCtrlItem->IsAskedForAnotherFile())
			{
				if (nDownloadDataRate==0)
					buffer="";
				else
					buffer.Format("%.1f %s", nDownloadDataRate/1024.0f,GetResString(IDS_KBYTESEC));
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			}
			break;

		case 4:		// file info
			{
				lpRect->bottom--; 
				lpRect->top++; 

				int iWidth = lpRect->right - lpRect->left; 
				int iHeight = lpRect->bottom - lpRect->top; 
				if (lpCtrlItem->m_Status == (HBITMAP)NULL)
					VERIFY(lpCtrlItem->m_Status.CreateBitmap(1, 1, 1, 8, NULL)); 
				CDC cdcStatus; 
				HGDIOBJ hOldBitmap;
				cdcStatus.CreateCompatibleDC(dc); 
				int cx = lpCtrlItem->m_Status.GetBitmapDimension().cx; 
				DWORD dwTicks = GetTickCount(); 
				if(lpCtrlItem->m_dwUpdated + DLC_BARUPDATE < dwTicks || cx !=  iWidth  || !lpCtrlItem->m_dwUpdated)
				{ 
					lpCtrlItem->m_Status.DeleteObject(); 
					lpCtrlItem->m_Status.CreateCompatibleBitmap(dc,  iWidth, iHeight); 
					lpCtrlItem->m_Status.SetBitmapDimension(iWidth,  iHeight); 
					hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->m_Status); 

					RECT rec_status; 
					rec_status.left = 0; 
					rec_status.top = 0; 
					rec_status.bottom = iHeight; 
					rec_status.right = iWidth; 
					try{
					lpClient->DrawStatusBar(&cdcStatus,  &rec_status,(lpCtrlItem->IsAskedForAnotherFile()), theApp.glob_prefs->UseFlatBar()); 
					}catch(...){} // in case client has been deleted meanwhile

					lpCtrlItem->m_dwUpdated = dwTicks + (rand() % 128); 
				} else 
					hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->m_Status); 

				dc->BitBlt(lpRect->left, lpRect->top, iWidth, iHeight,  &cdcStatus, 0, 0, SRCCOPY); 
				cdcStatus.SelectObject(hOldBitmap);

				lpRect->bottom++; 
				lpRect->top--; 
			}
			break;

	case 5:		// sources
			{
				// [Lord KiRon]
				// display client version
				if (nClientSoft == SO_UNKNOWN)
					buffer.Format("???");
				else
					buffer.Format("%s v%s",GetClientNameString(nClientSoft),GetClientVersionString(nClientSoft, nVersion));
			// [/Lord KiRon]
			dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
			break;
		}

		case 6:		// prio
		if ((bIsEmuleClient || nRemoteQueueRank)&&		//now emule v ? display the QR too!
			nDownloadState==DS_ONQUEUE)
		{
			if(bRemoteQueueFull){
				buffer=GetResString(IDS_QUEUEFULL);
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
			}
			else{
				if (nRemoteQueueRank){
				buffer.Format("QR: %u", nRemoteQueueRank);
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
			}
				else{
					buffer="";
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				}
			}
		} else {
					buffer="";
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
		}
			break;

		case 7:		// status
			// Lord KiRon
			RECT cur_rec;
			memcpy(&cur_rec, lpRect, sizeof(RECT));
			cur_rec.left += 6;
			point.x = cur_rec.left;
			point.y = cur_rec.top+1;

			dc->DrawText(sDownloadState,sDownloadState.GetLength(),&cur_rec, DLC_DT_TEXT);
			// Lord KiRon - end
			
		case 8:		// remaining time & size
			break;
		case 9: // UL/DL Ratio Maverick
			if (bIsRemoteInfoAvail || bCredits)
				buffer.Format("%0.1f", fRemoteBaseModifier);
			else buffer="";
			dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);	
			break;
		case 10: // Ratio total Maverick
			if (bIsRemoteInfoAvail || bCredits)
				buffer.Format("%ld", nRemoteRatio);
			else buffer="";
			dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);	
			break;
	}
	
	EMULE_CATCH
}

void CDownloadListCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	EMULE_TRY

	if (!lpDrawItemStruct->itemData)
		return;
	if(theApp.glob_prefs->DoUseSrcSortCol2() && theApp.glob_prefs->DoUseSort() && m_ShowSrc && m_sortTime+1000<::GetTickCount()) 
		SortInit(false); // Cax2 sortdlg
	CDC* odc = CDC::FromHandle(lpDrawItemStruct->hDC);
	CMuleCtrlItem* content = (CMuleCtrlItem*)lpDrawItemStruct->itemData;
	BOOL bCtrlFocused = ((GetFocus() == this) || (GetStyle() & LVS_SHOWSELALWAYS));

	if ((content->IsFile()) && (lpDrawItemStruct->itemAction | ODA_SELECT) && (lpDrawItemStruct->itemState & ODS_SELECTED))
	{
		if(bCtrlFocused)
			odc->SetBkColor(m_crHighlight);
		else
			odc->SetBkColor(m_crNoHighlight);
	} else
		odc->SetBkColor(GetBkColor());

	CMemDC dc(odc,&CRect(lpDrawItemStruct->rcItem));
	CFont *pOldFont = dc->SelectObject(GetFont());
	COLORREF crOldTextColor = dc->SetTextColor(m_crWindowText);

	BOOL notLast = lpDrawItemStruct->itemID + 1 != GetItemCount();
	BOOL notFirst = lpDrawItemStruct->itemID != 0;
	int tree_start;
	int tree_end;

	RECT cur_rec;
	memcpy(&cur_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));

	//offset was 4, now it's the standard 2 spaces
	int iOffset = dc->GetTextExtent(_T(" "), 1 ).cx * 2;
	CHeaderCtrl *pHeaderCtrl = GetHeaderCtrl();
	int iCount = pHeaderCtrl->GetItemCount();
	cur_rec.right = cur_rec.left;
	cur_rec.right -= iOffset;
	cur_rec.left += iOffset;

	if (content->IsFile())
	{		
		for(int iCurrent = 0; iCurrent < iCount; iCurrent++) 
		{
			int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);
			int cx = GetColumnWidth(iColumn);
			if(iColumn == 4)
			{
				int iNextLeft = cur_rec.left + cx;
				//set up tree vars
				cur_rec.left = cur_rec.right + iOffset;
				cur_rec.right = cur_rec.left + min(8, cx);
				tree_start = cur_rec.left + 1;
				tree_end = cur_rec.right;
				//normal column stuff
				cur_rec.left = cur_rec.right + 1;
				cur_rec.right = tree_start + cx - iOffset;
				DrawFileItem(dc, 4, &cur_rec, content);
				cur_rec.left = iNextLeft;
			} else {
				cur_rec.right += cx;
				DrawFileItem(dc, iColumn, &cur_rec, content);
				cur_rec.left += cx;
			}

		}
	} 
	else if (content->IsSource())
	{
		for(int iCurrent = 0; iCurrent < iCount; iCurrent++)
		{
			int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);
			int cx = GetColumnWidth(iColumn);

			if(iColumn == 4)
			{
				int iNextLeft = cur_rec.left + cx;
				//set up tree vars
				cur_rec.left = cur_rec.right + iOffset;
				cur_rec.right = cur_rec.left + min(8, cx);
				tree_start = cur_rec.left + 1;
				tree_end = cur_rec.right;
				//normal column stuff
				cur_rec.left = cur_rec.right + 1;
				cur_rec.right = tree_start + cx - iOffset;
				DrawSourceItem(dc, 4, &cur_rec, content);
				cur_rec.left = iNextLeft;
			} else {
				cur_rec.right += cx;
				DrawSourceItem(dc, iColumn, &cur_rec, content);
				cur_rec.left += cx;
			}
		}
	}

	//draw rectangle around selected item(s)
	if ((lpDrawItemStruct->itemAction | ODA_SELECT) &&
		(lpDrawItemStruct->itemState & ODS_SELECTED) && (content->IsFile()))
	{
		RECT outline_rec;
		memcpy(&outline_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));

		outline_rec.top--;
		outline_rec.bottom++;
		dc->FrameRect(&outline_rec, &CBrush(GetBkColor()));
		outline_rec.top++;
		outline_rec.bottom--;
		outline_rec.left++;
		outline_rec.right--;

		if(notFirst && (GetItemState(lpDrawItemStruct->itemID - 1, LVIS_SELECTED)))
		{
			CMuleCtrlItem* prev = (CMuleCtrlItem*)GetItemData(lpDrawItemStruct->itemID - 1);
			if(prev->IsFile())
				outline_rec.top--;
		} 

		if(notLast && (GetItemState(lpDrawItemStruct->itemID + 1, LVIS_SELECTED)))
		{
			CMuleCtrlItem* next = (CMuleCtrlItem*)GetItemData(lpDrawItemStruct->itemID + 1);
			if(next->IsFile())
				outline_rec.bottom++;
		} 

		if(bCtrlFocused)
			dc->FrameRect(&outline_rec, &CBrush(m_crFocusLine));
		else
			dc->FrameRect(&outline_rec, &CBrush(m_crNoFocusLine));
	}
	//draw focus rectangle around non-highlightable items when they have the focus
	else if (((lpDrawItemStruct->itemState & ODS_FOCUS) == ODS_FOCUS) &&
		(GetFocus() == this))
	{
		RECT focus_rec;
		focus_rec.top    = lpDrawItemStruct->rcItem.top;
		focus_rec.bottom = lpDrawItemStruct->rcItem.bottom;
		focus_rec.left   = lpDrawItemStruct->rcItem.left + 1;
		focus_rec.right  = lpDrawItemStruct->rcItem.right - 1;
		dc->FrameRect(&focus_rec, &CBrush(m_crNoFocusLine));
	}

	//draw tree last so it draws over selected and focus (looks better)
	if(tree_start < tree_end)
	{
		//set new bounds
		RECT tree_rect;
		tree_rect.top    = lpDrawItemStruct->rcItem.top;
		tree_rect.bottom = lpDrawItemStruct->rcItem.bottom;
		tree_rect.left   = tree_start;
		tree_rect.right  = tree_end;
		dc->SetBoundsRect(&tree_rect, DCB_DISABLE);

		//gather some information
		BOOL hasNext = notLast &&
			((CMuleCtrlItem*)GetItemData(lpDrawItemStruct->itemID + 1))->IsSource();
		BOOL isOpenRoot = hasNext && content->IsFile();
		BOOL isChild = content->IsSource();
		BOOL isExpandable = !isChild && content->GetFile()->GetSourceCount() > 0;
		//might as well calculate these now
		int treeCenter = tree_start + 3;
		int middle = (cur_rec.top + cur_rec.bottom + 1) / 2;

		//set up a new pen for drawing the tree
		CPen pn, *oldpn;
		pn.CreatePen(PS_SOLID, 1, dc->GetTextColor());
		oldpn = dc->SelectObject(&pn);

		if(isChild)
		{
			//draw the line to the status bar
			dc->MoveTo(tree_end, middle);
			dc->LineTo(tree_start + 3, middle);

			//draw the line to the child node
			if(hasNext)
			{
				dc->MoveTo(treeCenter, middle);
				dc->LineTo(treeCenter, cur_rec.bottom + 1);
			}
		} else if(isOpenRoot) {
			//draw circle
			RECT circle_rec;
			COLORREF crBk = dc->GetBkColor();
			circle_rec.top    = middle - 2;
			circle_rec.bottom = middle + 3;
			circle_rec.left   = treeCenter - 2;
			circle_rec.right  = treeCenter + 3;
			dc->FrameRect(&circle_rec, &CBrush(dc->GetTextColor()));
			dc->SetPixelV(circle_rec.left,      circle_rec.top,    crBk);
			dc->SetPixelV(circle_rec.right - 1, circle_rec.top,    crBk);
			dc->SetPixelV(circle_rec.left,      circle_rec.bottom - 1, crBk);
			dc->SetPixelV(circle_rec.right - 1, circle_rec.bottom - 1, crBk);
			//draw the line to the child node
			if(hasNext)
			{
				dc->MoveTo(treeCenter, middle + 3);
				dc->LineTo(treeCenter, cur_rec.bottom + 1);
			}
		}
		//draw the line back up to parent node
		if(notFirst && isChild)
		{
			dc->MoveTo(treeCenter, middle);
			dc->LineTo(treeCenter, cur_rec.top - 1);
		}

		//put the old pen back
		dc->SelectObject(oldpn);
		pn.DeleteObject();
	}

	//put the original objects back
	dc->SelectObject(pOldFont);
	dc->SetTextColor(crOldTextColor);
	EMULE_CATCH
}

BEGIN_MESSAGE_MAP(CDownloadListCtrl, CMuleListCtrl)
	ON_NOTIFY_REFLECT(LVN_ITEMACTIVATE, OnLvnItemActivate)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnListModified)
	ON_NOTIFY_REFLECT(LVN_INSERTITEM, OnListModified)
	ON_NOTIFY_REFLECT(LVN_DELETEITEM, OnListModified)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnNMRclick)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_NOTIFY_REFLECT(NM_CLICK, OnNMClick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnNMDblclkDownloadlist)
	ON_WM_SIZE()
	ON_WM_KEYUP()
END_MESSAGE_MAP()

void CDownloadListCtrl::OnLvnItemActivate(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	CMuleCtrlItem* content = (CMuleCtrlItem*)GetItemData(pNMIA->iItem);

	//--- xrmb:smartopen ---
	bool	isShift=GetAsyncKeyState(VK_SHIFT) < 0;
	bool	isCtrl=GetAsyncKeyState(VK_CONTROL) < 0;
	bool	isAlt=GetAsyncKeyState(VK_MENU) < 0;
	int		added=0;
	//--- :xrmb ---

	if (content->IsFile())
	{
		SetRedraw(false);
		if(!m_ShowSrc)		//Cax2 always restore the sort index that was there before opening the files...
			FilesSortIndex=m_lastSort; 
		m_ShowSrc=true;  //newsort added by Cax2 03/11/02 
		CPartFile* partfile = content->GetFile();
		
		if ( (!partfile->srcarevisible) && partfile->GetSourceCount()>0 && partfile->GetStatus()!=PS_COMPLETING)	//Cax2: when (slow) completing, the files stay shut
		{
			m_ItemMgr.LockRemove();
			MuleRange Range = m_ItemMgr.GetItemsByFile(partfile);
			for(MuleIter g = Range.first; g != Range.second; g++)
			{
				CMuleCtrlItem* pItem = (CMuleCtrlItem*)g->second;
				if (pItem->IsSource())
				{
					//--- xrmb:smartopen ---
					//--- if shift key only show ppl with transfer ---
					if(isShift || isCtrl || isAlt)
					{
						uint8 ds = pItem->GetSource()->GetDownloadState();
						if(!(isShift	&& ds==DS_DOWNLOADING ||
							isCtrl	&& pItem->GetSource()->GetRemoteQueueRank()>0 ||
							isAlt	&& ds!=DS_NONEEDEDPARTS))
								continue;
					}
					added++;
					//--- :xrmb ---
					InsertItem(LVIF_PARAM,pNMIA->iItem+1,0,0,0,0,(LPARAM)pItem);
				}
			}
			m_ItemMgr.UnlockRemove();
			//--- xrmb:smartopen ---
			partfile->srcarevisible = added!=0;
			//--- :xrmb ---
		} else {
			HideSources(partfile); //Cax2 closefile bugfix [thx Lord KiRon] 
		}
		SortInit(partfile->srcarevisible&&theApp.glob_prefs->DoUseSort()); //override if useSort & showing sources...
		//newsort end 
		SetRedraw(true);
	}
	*pResult = 0;
	EMULE_CATCH
}

void CDownloadListCtrl::OnNMRclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	POINT point;
	GetCursorPos(&point);
	if (GetSelectionMark() != (-1))
	{
		CMuleCtrlItem* content = (CMuleCtrlItem*)GetItemData(GetSelectionMark());
		const CPartFile* cFile=NULL; //Cax2 bugfix for Dr Slump? start
		if (content->IsFile()) cFile = content->GetFile();
		if (cFile!=NULL)
		{
			CPartFile* file=(CPartFile*)cFile; //Cax2 bugfix
			bool filedone=(file->GetStatus()==PS_COMPLETE || file->GetStatus()==PS_COMPLETING);
			bool justOne=(GetSelectedCount()==1); //Cax2 faster code

			theApp.emuledlg->transferwnd.infolistctrl.SwitchToFile(file); // FoRcHa
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
			
			m_FileMenu.EnableMenuItem(MP_PAUSE,((file->GetStatus() != PS_PAUSED && file->GetStatus() != PS_ERROR && !filedone) ? MF_ENABLED:MF_GRAYED));
			m_FileMenu.EnableMenuItem(MP_STOP,((file->GetStatus() != PS_ERROR && !filedone ) ? MF_ENABLED:MF_GRAYED));
			m_FileMenu.EnableMenuItem(MP_RESUME,((file->GetStatus() == PS_PAUSED) ? MF_ENABLED:MF_GRAYED));
			m_FileMenu.EnableMenuItem(MP_OPEN,((justOne && file->GetStatus() == PS_COMPLETE) ? MF_ENABLED:MF_GRAYED)); //<<--9/21/02
			m_FileMenu.EnableMenuItem(MP_PREVIEW,((justOne && file->PreviewAvailable()) ? MF_ENABLED:MF_GRAYED));
			
			if(theApp.glob_prefs->SLSEnable())
			{
				m_FileMenu.EnableMenuItem(MP_LOADSOURCES,((file->GetStatus() != PS_COMPLETE && file->GetStatus() != PS_COMPLETING)?MF_ENABLED:MF_GRAYED));
				m_FileMenu.EnableMenuItem(MP_SAVESOURCES,MF_ENABLED);
			} else {
				m_FileMenu.EnableMenuItem(MP_LOADSOURCES,MF_GRAYED);
				m_FileMenu.EnableMenuItem(MP_SAVESOURCES,MF_GRAYED);
			}			

			m_FileMenu.EnableMenuItem(MP_DROPNONEEDEDSRCS,MF_ENABLED); // Added by Tarod
			m_FileMenu.EnableMenuItem(MP_ALL_A4AF_TO_HERE,(justOne)?MF_ENABLED:MF_GRAYED); // Tarod(+Cax2 Patch 21/12/02)
			m_FileMenu.EnableMenuItem(MP_METINFO,(justOne)?MF_ENABLED:MF_GRAYED); // Cax2
			m_FileMenu.EnableMenuItem(MP_VIEWFILECOMMENTS,(justOne)?MF_ENABLED:MF_GRAYED); // Cax2
			m_FileMenu.EnableMenuItem(MP_FAKECHECK1,(justOne)?MF_ENABLED:MF_GRAYED); // DeltaHF (+Cax2 Patch 21/12/02)
			if (justOne) // Cax2
			{
				m_FileMenu.CheckMenuItem(MP_ALL_A4AF_AUTO, file->IsA4AFAuto()? MF_CHECKED: MF_UNCHECKED); // Tarod
				m_FileMenu.EnableMenuItem(MP_ALL_A4AF_AUTO, MF_ENABLED);
			} else {
				m_FileMenu.CheckMenuItem(MP_ALL_A4AF_AUTO, MF_UNCHECKED);
				m_FileMenu.EnableMenuItem(MP_ALL_A4AF_AUTO, MF_GRAYED);
			}
			m_FileMenu.EnableMenuItem(MP_CANCEL,(!filedone) ? MF_ENABLED:MF_GRAYED);

			if( (file->IsMovie()||file->IsArchive()) && justOne)
			{
				m_FileMenu.EnableMenuItem(MP_MOVIE, MF_ENABLED);
				m_FileMenu.CheckMenuItem(MP_MOVIE, (file->GetMovieMode() != 0)? MF_CHECKED:MF_UNCHECKED);
			} else
				m_FileMenu.EnableMenuItem(MP_MOVIE, MF_GRAYED);

			m_PrioMenu.CheckMenuItem(MP_PRIOAUTO, (justOne && file->IsAutoPrioritized())? MF_CHECKED:MF_UNCHECKED);
			m_PrioMenu.CheckMenuItem(MP_PRIOHIGH,(justOne && !file->IsAutoPrioritized() && file->GetPriority() == PR_HIGH)? MF_CHECKED:MF_UNCHECKED);
			m_PrioMenu.CheckMenuItem(MP_PRIONORMAL,(justOne && !file->IsAutoPrioritized() && file->GetPriority() == PR_NORMAL)? MF_CHECKED:MF_UNCHECKED);
			m_PrioMenu.CheckMenuItem(MP_PRIOLOW,(justOne && !file->IsAutoPrioritized() && file->GetPriority() == PR_LOW)? MF_CHECKED:MF_UNCHECKED);

			m_FileMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
		} else {
			const CUpDownClient* client = content->GetSource();
			if (client !=NULL) 
				theApp.emuledlg->transferwnd.infolistctrl.SwitchToUser((CUpDownClient*)client);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();

			m_ClientMenu.CreatePopupMenu();
			m_ClientMenu.AddMenuTitle(GetResString(IDS_CLIENTS));
			m_ClientMenu.AppendMenu(MF_STRING,MP_DETAIL, GetResString(IDS_SHOWDETAILS));
			m_ClientMenu.AppendMenu(MF_STRING,MP_ADDFRIEND, GetResString(IDS_ADDFRIEND));
			m_ClientMenu.AppendMenu(MF_STRING,MP_MESSAGE, GetResString(IDS_SEND_MSG));
			m_ClientMenu.AppendMenu(MF_STRING,MP_SHOWLIST, GetResString(IDS_VIEWFILES));
			if(content->IsAskedForAnotherFile())
				m_ClientMenu.AppendMenu(MF_STRING,MP_DOWNNOW, GetResString(IDS_DOWNLOAD_A4AF)); // Ambdribant
			m_ClientMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
			m_ClientMenu.DestroyMenu();
		}
	} else {
		m_FileMenu.EnableMenuItem((UINT_PTR)m_PrioMenu.m_hMenu,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_CANCEL,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_PAUSE,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_STOP,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_RESUME,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_PREVIEW,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_OPEN,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_FAKECHECK1,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_MOVIE,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_VIEWFILECOMMENTS,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_METINFO,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_LOADSOURCES,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_SAVESOURCES,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_DROPNONEEDEDSRCS,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_ALL_A4AF_TO_HERE,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_ALL_A4AF_AUTO,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_GETED2KLINK,MF_GRAYED);
		m_FileMenu.EnableMenuItem(MP_GETHTMLED2KLINK,MF_GRAYED);
		m_FileMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
	}
	*pResult = 0;

	EMULE_CATCH
}

BOOL CDownloadListCtrl::OnCommand(WPARAM wParam,LPARAM lParam )
{
	try
	{

	if (GetSelectionMark() != (-1))
	{
		CMuleCtrlItem* content = (CMuleCtrlItem*)GetItemData(GetSelectionMark());
		
		// for multiple selections 
		UINT selectedCount = GetSelectedCount(); 
		CTypedPtrList<CPtrList, CPartFile*> selectedList; 
		int index = -1; 
		POSITION pos = GetFirstSelectedItemPosition(); 
		while(pos != NULL) 
		{ 
			index = GetNextSelectedItem(pos); 
			if(index > -1) 
			{
				if (((CMuleCtrlItem*)GetItemData(index))->IsFile())
					selectedList.AddTail(((CMuleCtrlItem*)GetItemData(index))->GetFile());
			} 
		} 

		if (content->IsFile())
		{
			CPartFile* file = content->GetFile();

			switch (wParam) //Cax2 note: all multiple/single selection code is now merged into one! [thx LordKiRon]
			{
				case MP_CANCEL:
					{
						CString fileList; 
						for (pos = selectedList.GetHeadPosition() ; pos != 0 ; selectedList.GetNext(pos))
						{
							fileList += "\n" ; 
							fileList += selectedList.GetAt(pos)->GetFileName(); 
						} 
						CString str=(selectedCount > 1)?GetResString(IDS_Q_CANCELDL):GetResString(IDS_Q_CANCELDL2);	//Cax2 bugfix: plurals
						if (MessageBox(str + fileList,GetResString(IDS_CANCEL),MB_ICONQUESTION|MB_YESNO) == IDYES)
						{ 
							while(!selectedList.IsEmpty())
							{
								switch(selectedList.GetHead()->GetStatus())
								{ 
									case PS_WAITINGFORHASH: 
									case PS_HASHING: 
									case PS_COMPLETING: 
									case PS_COMPLETE: 
										selectedList.RemoveHead(); 
										break; 
									default: 
										selectedList.GetHead()->DeleteFile(); 
										selectedList.RemoveHead();
								}
							}  
						}
						break; 
					}
				case MP_PRIOHIGH:
					while(!selectedList.IsEmpty())
					{ 
						selectedList.GetHead()->SetAutoPriority(false); // Tarod
						selectedList.GetHead()->SetPriority(PR_HIGH); 
						selectedList.RemoveHead();
					} 
					break; 
				case MP_PRIOLOW:
					while(!selectedList.IsEmpty())
					{ 
						selectedList.GetHead()->SetAutoPriority(false); // Tarod
						selectedList.GetHead()->SetPriority(PR_LOW); 
						selectedList.RemoveHead();
					} 
					break; 
				case MP_PRIONORMAL:
					while(!selectedList.IsEmpty())
					{ 
						selectedList.GetHead()->SetAutoPriority(false); // Tarod
						selectedList.GetHead()->SetPriority(PR_NORMAL); 
						selectedList.RemoveHead();
					} 
					break; 
				case MP_PRIOAUTO: // Added by Tarod
					while(!selectedList.IsEmpty())
					{ 
						selectedList.GetHead()->SetAutoPriority(true);  // Added by Tarod
						selectedList.GetHead()->SetPriority(PR_NORMAL); 
						selectedList.RemoveHead();
					} 
					break; 
				case MP_PAUSE:
					while(!selectedList.IsEmpty())
					{ 
						selectedList.GetHead()->PauseFile(); 
						selectedList.RemoveHead();
					} 
					break; 
				case MP_RESUME:
					while(!selectedList.IsEmpty())
					{ 
						selectedList.GetHead()->ResumeFile(); 
                        selectedList.GetHead()->SavePartFile();
						selectedList.RemoveHead();
					} 
					break; 
				case MP_STOP:
					while(!selectedList.IsEmpty())
					{ 
                        HideSources(selectedList.GetHead());
						selectedList.GetHead()->StopFile(); 
						selectedList.RemoveHead();
					} 
					break; 
				case MP_CLEARCOMPLETED:
					ClearCompleted();
					break;
				case MP_METINFO:
					{
						CFileDetailDialog dialog(file);
						dialog.DoModal();
					}
					break;
				case MP_GETED2KLINK:
					{
						CString str;
						while(!selectedList.IsEmpty())
						{ 
							str += theApp.CreateED2kLink(selectedList.GetHead()) + "\n"; 
							selectedList.RemoveHead();
						}
						theApp.CopyTextToClipboard(str);
						//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
					}
					break;
				case MP_GETHTMLED2KLINK:
					{
						CString str;
						while(!selectedList.IsEmpty())
						{ 
							str += theApp.CreateHTMLED2kLink(selectedList.GetHead()) + "\n"; 
							selectedList.RemoveHead();
						}
						theApp.CopyTextToClipboard(str);
						//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
					}
					break;
				case MP_OPEN:
					{
						if(selectedCount > 1) break;
						char* buffer = new char[250];
						sprintf(buffer,"%s\\%s",theApp.glob_prefs->GetIncomingDir(),file->GetFileName());
						ShellOpenFile(buffer);
						delete buffer;
					}
					break;
				case MP_PREVIEW:
					{
						if(selectedCount > 1) break;
						file->PreviewFile();
					}
					break;
				case MP_VIEWFILECOMMENTS:
					{
						CCommentDialogLst dialog(file); 
						dialog.DoModal(); 
					}
					break;
#ifdef OLD_SLS_ENABLED
				case MP_LOADSOURCES:	// Mr.Fry
					while(!selectedList.IsEmpty())
					{
						if (theApp.glob_prefs->IsRemoveNNSEnabled()) // Vorlost added prefs question
							selectedList.GetHead()->RemoveNoNeededSources(); // Tarod - clean no needed sources
						//Vorlost (Patch by Mr.Fry)
						if(theApp.downloadqueue->KeepStoredSourcesOpen())
							selectedList.GetHead()->AddFromStoredSources(selectedList.GetHead()->GetStoredSources()
												     ,true
												     ,selectedList.GetHead()->GetLoadSourcesAtOnceLimit()
												     ,true);
						else
							selectedList.GetHead()->LoadAndAddSources(false
												  ,true
												  ,file->GetLoadSourcesAtOnceLimit()
												  ,true);
						selectedList.RemoveHead();
					}
				case MP_SAVESOURCES:	// Mr.Fry
					while(!selectedList.IsEmpty())
					{
						selectedList.GetHead()->SaveSources();
						selectedList.RemoveHead();
					}
					break;
#endif //OLD_SLS_ENABLED
                case MP_ALL_A4AF_TO_HERE: // Tarod
					{
						DownloadAllA4AF(file);
					}
					break;
				case MP_ALL_A4AF_AUTO: // Tarod
					{
						if (file->IsA4AFAuto())
						{
							file->SetA4AFAuto(false);
						} else {
							theApp.downloadqueue->DisableAllA4AFAuto();
							file->SetA4AFAuto(true);
						}
					}
					break;
				case MP_DROPNONEEDEDSRCS:
					while(!selectedList.IsEmpty()) // Tarod
					{
						selectedList.GetHead()->RemoveNoNeededSources();
						selectedList.RemoveHead();
					} 
					break;
           		case MP_FAKECHECK1:	// DeltaHF
					theApp.Action(file,NULL,NULL);
					break;
				// DonGato - Moview Preview Mode
				case MP_MOVIE:
					if (file->GetMovieMode())
					{
						file->SetMovieMode(0);
						file->SaveMovieMode();
					} else {
						if (file->IsAviMovie())
						{
							if (selectedList.GetHead()->GetFileSize() < 209715200)
								file->SetMovieMode(1);
							else
								file->SetMovieMode(2);
						}	
						else if (file->IsMpgMovie())
						{
							if (selectedList.GetHead()->GetFileSize() < 209715200)
								file->SetMovieMode(4);
							else
								file->SetMovieMode(5);
						}
						else
							file->SetMovieMode(0);
						file->SaveMovieMode();
					}						
					break;
				// DonGato - Moview Preview Mode end
			}
		} else {
			CUpDownClient* client = content->GetSource();
			switch (wParam)
			{
				case MP_SHOWLIST:
					client->RequestSharedFileList();
					break;
				case MP_DOWNNOW: 
					client->HandoverToAnotherFile(content->GetFile()); 
					break; 
				case MP_MESSAGE:
					theApp.emuledlg->chatwnd.StartSession(client);
					break;
				case MP_ADDFRIEND:
					theApp.friendlist->AddFriend(client);
					break;
				case MP_DETAIL:
					CClientDetailDialog dialog(client);
					dialog.DoModal();
					break;
			}
		}
		// cleanup multiselection
		selectedList.RemoveAll(); 
	}
	else /*nothing selected*/
	{
		switch (wParam)
		{
			case MP_CLEARCOMPLETED:
				ClearCompleted();
				break;
		}

	}

	return true;

	}catch(...){}

	return false;
}

void CDownloadListCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult)
{
	EMULE_TRY
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	m_SortAscending = (m_CurrentSortItem != pNMListView->iSubItem) ? true : !m_SortAscending;
	m_CurrentSortItem = pNMListView->iSubItem;
	SetSortArrow(m_CurrentSortItem, m_SortAscending);
	m_lastSort=m_CurrentSortItem + (m_SortAscending ? 0:10); // Cax2
	SortItems(SortProc, m_CurrentSortItem + (m_SortAscending ? 0:10)+(m_ShowSrc?1000:0)); // Cax2
	EMULE_CATCH
	*pResult = 0;
}

int CDownloadListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	EMULE_TRY

	CMuleCtrlItem* item1 = (CMuleCtrlItem*)lParam1;
	CMuleCtrlItem* item2 = (CMuleCtrlItem*)lParam2;

	bool showSrc=false;  
	if (lParamSort>=1000)
	{ 
		showSrc=true; 
		lParamSort-=1000; 
	} 
    int sortMod = 1;
    if(lParamSort >= 10)
    {
		sortMod = -1;
		lParamSort -= 10;
	}

	int comp;
	if ((item1->IsFile() && !item2->IsFile())||(item2->IsFile() && !item1->IsFile())){ // files vs source?
		comp=0; // leave it as it is...
	} else if (item1->IsFile()) { //both files? (!)
		if (showSrc)
			comp= 0;
		else
		{ 
			CPartFile* file1 = item1->GetFile();
			CPartFile* file2 = item2->GetFile();
			comp = Compare(file1, file2, lParamSort);
		}
    } else { //both sources...
		comp = Compare(item1->GetFile(), item2->GetFile(),0);
        if(comp != 0) return 0;
		//Cax2:a4af clients kept separate, unless sorting for name
		if ((item1->IsAskedForAnotherFile() || item2->IsAskedForAnotherFile()) && lParamSort!=0)
		{
			int n = 0;
			if(item1->IsAskedForAnotherFile() && !item2->IsAskedForAnotherFile())
				n = -1;
			if(item2->IsAskedForAnotherFile() && !item1->IsAskedForAnotherFile())
				n = 1;
			if (n!=0)			//always at the end, unless sort by source - then a4af always at the top
				return (lParamSort==5)? n : -n;
		}
		comp = Compare(item1->GetSource(), item2->GetSource(), lParamSort, sortMod);
    } 
   	return sortMod * comp;

	EMULE_CATCH
	return 0;
}

void CDownloadListCtrl::HideSources(CPartFile* partfile)
{
	EMULE_TRY
	if (!partfile->srcarevisible)
		return; 
	partfile->srcarevisible = false;
	SetRedraw(false); 
	CMuleCtrlItem* pItem; 
    LVFINDINFO find;
    find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)m_ItemMgr.FindFile(partfile);
    int nItem = FindItem(&find);
	if(nItem < 0) nItem = 0;
	for (int i = nItem; i <GetItemCount(); )
	{ 
		pItem = (CMuleCtrlItem*)GetItemData(i); 
		if(pItem->IsSource() && pItem->GetFile() == partfile)
		{
			pItem->m_dwUpdated = 0; 
			pItem->m_Status.DeleteObject(); 
			DeleteItem(i);
		}
		else
			i++;
	} 
	if (m_ShowSrc)			//Cax2 check if we don't show sources anymore...
	{
		m_ShowSrc=false; 
		for(int i = 0; i < GetItemCount(); i++)
			if(((CMuleCtrlItem*)GetItemData(i))->IsSource()) 
			{
				m_ShowSrc=true;
				break;
			}
		if (!m_ShowSrc)	//Cax2 always restore the sort index that was there before expanding the files...
		{
			m_lastSort=FilesSortIndex;	
			m_CurrentSortItem=(m_lastSort<10)?m_lastSort:m_lastSort-10;
			SetSortArrow(m_CurrentSortItem,(m_lastSort<10));
		}
	}
	SetRedraw(true);
	EMULE_CATCH
}

void CDownloadListCtrl::ClearCompleted()
{
	EMULE_TRY
	
	bool bFound = true;
	while(bFound)
	{
		bFound = false;
		m_ItemMgr.LockRemove();
		MuleRange Range = m_ItemMgr.GetAllFiles();
		for(MuleIter g = Range.first; g != Range.second; g++)
		{
			CPartFile* file = (CPartFile*)g->second;
			if (!file->IsPartFile())
			{
				m_ItemMgr.UnlockRemove();
				RemoveFile(file);
				bFound = true;
				break;
			}
		}
		if(!bFound)
			m_ItemMgr.UnlockRemove();
	}

	EMULE_CATCH
}

void CDownloadListCtrl::SetStyle() 
{
	EMULE_TRY
	if (theApp.glob_prefs->IsDoubleClickEnabled())
		SetExtendedStyle(LVS_EX_FULLROWSELECT);
	else
		SetExtendedStyle(LVS_EX_ONECLICKACTIVATE | LVS_EX_FULLROWSELECT);
	EMULE_CATCH
}

void CDownloadListCtrl::SortInit(int override) // Cax2 sortdlg
{
	EMULE_TRY
	m_sortTime=::GetTickCount();
	if (override) m_lastSort-=(m_lastSort>=0)?10000:0;	//simple override, keep the old sorting column, restore it later...
	if (override==99) //major override, wipe old sorting order...
	{
		m_lastSort=theApp.glob_prefs->GetDownloadSortCol();
		m_CurrentSortItem=(m_lastSort<10)?m_lastSort:m_lastSort-10;
		SetSortArrow(m_CurrentSortItem,(m_lastSort<10));
	}
	if(m_ShowSrc)
	{
		if(theApp.glob_prefs->DoUseSort() && m_lastSort<0)
		{
			SortItems(&SortProc,1000 + theApp.glob_prefs->GetSrcSortCol1()); //+ show sources(clients)
			if (theApp.glob_prefs->DoUseSrcSortCol2())
			{
				if (theApp.glob_prefs->GetSrcSortCol2()==255)
				{
					SortItems(&SortProc,1017); // kiron's status desc, show sources(clients) 05/01/03
					SortItems(&SortProc,1013); // speed desc, show sources(clients)
				} else 
					SortItems(&SortProc,1000 + theApp.glob_prefs->GetSrcSortCol2());
			}
		}
	} else {
		if (m_lastSort<0)
		{
			SortItems(&SortProc,theApp.glob_prefs->GetDownloadSortCol());
		} else 
			SortItems(&SortProc, m_lastSort);
	}
	EMULE_CATCH
}
void CDownloadListCtrl::OnListModified(NMHDR *pNMHDR, LRESULT *pResult) 
{
	EMULE_TRY

	NM_LISTVIEW *pNMListView = (NM_LISTVIEW*)pNMHDR;

	//this works because true is equal to 1 and false equal to 0
	BOOL notLast = pNMListView->iItem + 1 != GetItemCount();
	BOOL notFirst = pNMListView->iItem != 0;

	// Lord KiRon
	CRect rcViewRect;
	GetViewRect(&rcViewRect);
	CRect rcItemRect;
	GetItemRect(pNMListView->iItem,&rcItemRect,LVIR_BOUNDS);
	CRect rcIntersection;
	rcIntersection.IntersectRect(rcViewRect,rcItemRect);
	if (!rcIntersection.IsRectEmpty())
	{
		RedrawItems(pNMListView->iItem - notFirst, pNMListView->iItem + notLast);
	}
	// Lord KiRon - end
	//RedrawItems(pNMListView->iItem - notFirst, pNMListView->iItem + notLast);

	EMULE_CATCH
}

int CDownloadListCtrl::Compare(CPartFile* file1, CPartFile* file2, LPARAM lParamSort) 
{
	EMULE_TRY
	if (file1==NULL||file2==NULL)
		return 0;
	switch(lParamSort)
	{
		case 0: //filename asc
            // obaldin: funny, but we might not have a filename yet..
            {
                char *fname1=file1->GetFileName();
                char *fname2=file2->GetFileName();
                if  (!fname1||!fname2)
                    return 0;
                else
			        return strcmpi(fname1,fname2);
            }
		case 1: //size asc
			return (file1->GetFileSize()== file2->GetFileSize())?0:((file1->GetFileSize() < file2->GetFileSize())?-1:1);		//sort for >2gb
		case 2: //transfered asc
			return file1->GetTransfered() - file2->GetTransfered();
		case 3: //speed asc
			return file1->GetDatarate() - file2->GetDatarate();
		case 4: //progress asc
			{
				float comp = file1->GetPercentCompleted() - file2->GetPercentCompleted();
				return (comp > 0)?1:((comp<0)?-1:0);
			}
		case 5: //sources asc
			return file1->GetSourceCount() - file2->GetSourceCount();
		case 6: //priority asc
			return file1->GetPriority() - file2->GetPriority();
		case 7: //Status asc 
			return file1->GetPartfileStatus().Compare(file2->GetPartfileStatus()); 
		case 8: //Remaining Time asc
			{
				int tmp1=file1->GetTimeRemaining();
				int tmp2=file2->GetTimeRemaining();
				if (tmp1>0&&tmp2>0) 
					return tmp1-tmp2 ;
				if (tmp1>0||tmp2>0)
					return tmp2-tmp1; 
				//if no remaining time sort by remaining size asc. Leave completed files last...
				tmp1=(file1->GetStatus()==PS_COMPLETING || file1->GetStatus()==PS_COMPLETE);
				tmp2=(file2->GetStatus()==PS_COMPLETING || file2->GetStatus()==PS_COMPLETE);
				if ((tmp1) && (tmp2))
					return 0;
				if (tmp2)
					return -1;
				if (tmp1)
					return 1;
				tmp1=file1->GetFileSize()-file1->GetCompletedSize();
				tmp2=file2->GetFileSize()-file2->GetCompletedSize();
				return (tmp1==tmp2)?0:((tmp1<tmp2)?-1:1); 
			}
		default:
			return 0;
	}
	EMULE_CATCH

	return 0;
}

int CDownloadListCtrl::Compare(CUpDownClient* client1,CUpDownClient* client2, LPARAM lParamSort, int sortMod) 
{
	EMULE_TRY
	if (client1==NULL||client2==NULL) return 0;
		switch(lParamSort)
		{
			case 0: //name asc
				{
				CString sName1, sName2;
				try{ sName1 = CString(client1->GetUserName()); }
				catch(...){ sName1 = _T(""); }
				try{ sName2 = CString(client2->GetUserName()); }
				catch(...){ sName2 = _T(""); }
				if  (sName1.IsEmpty() || sName2.IsEmpty())
					return (sName1.IsEmpty()) ? ((sName2.IsEmpty()) ? 0:sortMod):-1*sortMod;	//Cax2 append all unknown ones at the end
				return strcmpi(sName1, sName2);
				}
			case 2: //transfered asc
				return client1->GetTransferedDown() - client2->GetTransferedDown();
			case 3: //speed asc
				return client1->GetDownloadDatarate() - client2->GetDownloadDatarate();
			case 4: //progress asc
				return client1->GetAvailablePartCount() - client2->GetAvailablePartCount();
			case 5: //sources column
				if(client1->GetClientSoft() == SO_UNKNOWN || client2->GetClientSoft() == SO_UNKNOWN )
					return (client1->GetClientSoft()-client2->GetClientSoft())*sortMod;		//Cax2 append all unknown ones at the end
				if(client1->GetClientSoft() == client2->GetClientSoft())
					if(client1->IsEmuleClient())
					{
						// Lord KiRon - not needed any more because version corrected when read
						/*if ((client2->GetMuleVersion()==0x2B  &&  client1->GetMuleVersion()==0x22)||(client1->GetMuleVersion()==0x2B  &&  client2->GetMuleVersion()==0x22))
								return client2->GetMuleVersion() - client1->GetMuleVersion();
						return ((client2->GetMuleVersion()==0x2B)?0x22:client2->GetMuleVersion()) - ((client1->GetMuleVersion()==0x2B)?0x22:client1->GetMuleVersion());*/
						return client2->GetMuleVersion() - client1->GetMuleVersion();
					}
					else
						return client2->GetVersion() - client1->GetVersion();
				return client1->GetClientSoft() - client2->GetClientSoft();
			case 6: //priority column - QR 
				if (client1->GetDownloadState()!=DS_ONQUEUE || client2->GetDownloadState()!=DS_ONQUEUE)	//Cax2 QR gaps bugfix
				{
					if(client1->GetClientSoft() == SO_UNKNOWN || client2->GetClientSoft() == SO_UNKNOWN )		//Cax2 append all unknown ones at the end
								return (client1->GetClientSoft()-client2->GetClientSoft())*sortMod;							//Cax2 append all unknown ones at the end
					if (client1->GetDownloadState()==DS_DOWNLOADING || client2->GetDownloadState()==DS_DOWNLOADING)		//Cax2 QR gaps bugfix
						return (client1->GetDownloadState()==DS_DOWNLOADING)?((client2->GetDownloadState()==DS_DOWNLOADING)?0:-1):2; //Cax2 QR gaps bugfix
					return (client1->GetDownloadState()!=DS_ONQUEUE)?((client2->GetDownloadState()!=DS_ONQUEUE)?0:sortMod):-1*sortMod;		//Cax2 now should show properly...
				}
				if (sortMod<0 || (client1->GetRemoteQueueRank()>0 && client2->GetRemoteQueueRank()>0))
					return client1->GetRemoteQueueRank() - client2->GetRemoteQueueRank();   
				if(client1->GetRemoteQueueRank()==0 && client2->GetRemoteQueueRank()==0)
					return (client1->IsEmuleClient())?((client2->IsEmuleClient())?0:-1):2;	//"queue full" or " " ?
				return (client1->GetRemoteQueueRank()==0)?((client2->GetRemoteQueueRank()==0)?0:2):-1;  
			case 7: //Status column - Lord KiRon
				return client1->GetDownloadStateAsString().Compare(client2->GetDownloadStateAsString()); 
			case 9: // UL/DL column - Maverick
				if ((client1->IsRemoteInfoAvaiable() || client1->credits) &&
					(client2->IsRemoteInfoAvaiable() || client2->credits))
					return (client1->GetRemoteBaseModifier() - client2->GetRemoteBaseModifier())*10;		//Cax2 - has to return an integer!
				if (client1->IsRemoteInfoAvaiable() || client1->credits)				
					return 1;
				if (client2->IsRemoteInfoAvaiable() || client2->credits)				
					return -1;
				return 0;
			case 10: // Ratio column - Maverick
				if ((client1->IsRemoteInfoAvaiable() || client1->credits) &&
					(client2->IsRemoteInfoAvaiable() || client2->credits))
					return client1->GetRemoteRatio() - client2->GetRemoteRatio();
				if (client1->IsRemoteInfoAvaiable() || client1->credits)				
					return 1;
				if (client2->IsRemoteInfoAvaiable() || client2->credits)				
					return -1;
				return 0;
			default:
				return 0;
		}
	EMULE_CATCH
	return 0;
}

void CDownloadListCtrl::OnNMClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY
	
	RefreshInfo();

	if(theApp.glob_prefs->IsDoubleClickEnabled())	// pretty useless/faulty if single-click 
	{												// for opening is activated
		DWORD pos = GetMessagePos();
		CPoint pt(LOWORD(pos), HIWORD(pos));
		ScreenToClient(&pt);

		// Get indexes of the first and last visible items in 
		// the listview control.
		int index = GetTopIndex();
		int last_visible_index = index + GetCountPerPage();
		if(last_visible_index > GetItemCount())
			last_visible_index = GetItemCount();

		// Loop until number visible items has been reached.
		while(index <= last_visible_index)
		{
			// Get the bounding rectangle of an item. If the mouse
			// location is within the bounding rectangle of the item,
			// you know you have found the item that was being clicked.
			CRect r;
			GetItemRect(index, &r, LVIR_BOUNDS);
			if(r.PtInRect(pt))
			{
				CMuleCtrlItem *pListItem = (CMuleCtrlItem*)GetItemData(index);
				if(!pListItem)
					return;
				if(!pListItem->IsFile())			// if it isn't a file we're done
					return;

			// [+]/[-]	
				CRect rPlusMinus(r);
				rPlusMinus.left += 4 + 2;			// eeek, hardcoded values
				rPlusMinus.top += 1 + 2;						//  --||--
				rPlusMinus.right = rPlusMinus.left + 16 - 4;	//  --||--
				rPlusMinus.bottom = rPlusMinus.top + 16 - 4;	//  --||--
				if(rPlusMinus.PtInRect(pt))
					OnLvnItemActivate(pNMHDR, pResult);
			// rating
				CRect rRating;
				rRating.left = rPlusMinus.right;
				rRating.top = rPlusMinus.top;
				rRating.right = rRating.left + 16 - 4;
				rRating.bottom = rRating.top + 16 - 2;
				if(rRating.PtInRect(pt))
				{						
					CPartFile *pPartFile = pListItem->GetFile();
					if(!pPartFile)
						return;
					if(pPartFile->HasComment())
					{	
						CCommentDialogLst dlg(pPartFile); 
						dlg.DoModal(); 
					}
                }
				break;
			}

			// Get the next item in listview control.
			index++;
		}
	}	
	
	*pResult = 0;

	EMULE_CATCH
}

void CDownloadListCtrl::OnNMDblclkDownloadlist(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	if (GetSelectionMark() != (-1))
	{
		CMuleCtrlItem* content = (CMuleCtrlItem*)GetItemData(GetSelectionMark());

		if (content->IsFile())		//Cax2: doubleclick opens completed files
		{
			CPartFile* file = content->GetFile();
			if(file->GetStatus()==PS_COMPLETE)
			{
				char* buffer = new char[MAX_PATH];
				sprintf(buffer,"%s\\%s",file->GetPath(),file->GetFileName());
				theApp.emuledlg->AddLogLine( false, "%s\\%s",file->GetPath(),file->GetFileName());
				ShellOpenFile(buffer);
				delete buffer;
			}
		}
		else	//client  options
		{
			CUpDownClient* client = content->GetSource();
			if (theApp.glob_prefs->GetDetailsOnClick())
			{
				CClientDetailDialog dialog(client);
				dialog.DoModal();
			} else {
				theApp.emuledlg->chatwnd.StartSession(client);
			}
		}
	}
	
	*pResult = 0;

	EMULE_CATCH
}

void CDownloadListCtrl::CreateMenues() 
{
	EMULE_TRY

	char buffer[100];
	if (m_PrioMenu) m_PrioMenu.DestroyMenu();
	if (m_FileMenu) m_FileMenu.DestroyMenu();

	m_PrioMenu.CreateMenu();
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOLOW, GetResString(IDS_PRIOLOW));
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIONORMAL,GetResString(IDS_PRIONORMAL));
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOHIGH, GetResString(IDS_PRIOHIGH));
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOAUTO, GetResString(IDS_PRIOAUTO));

	m_FileMenu.CreatePopupMenu();
	m_FileMenu.AddMenuTitle(GetResString(IDS_DOWNLOADMENUTITLE));

	m_FileMenu.AppendMenu(MF_STRING,MP_CLEARCOMPLETED, GetResString(IDS_DL_CLEAR));
	m_FileMenu.AppendMenu(MF_SEPARATOR);
	m_FileMenu.AppendMenu(MF_STRING|MF_POPUP,(UINT_PTR)m_PrioMenu.m_hMenu, GetResString(IDS_PRIORITY));
	m_FileMenu.AppendMenu(MF_STRING,MP_CANCEL, GetResString(IDS_MAIN_BTN_CANCEL));
	m_FileMenu.AppendMenu(MF_STRING,MP_STOP, GetResString(IDS_DL_STOP));
	m_FileMenu.AppendMenu(MF_STRING,MP_PAUSE, GetResString(IDS_DL_PAUSE));
	m_FileMenu.AppendMenu(MF_STRING,MP_RESUME, GetResString(IDS_DL_RESUME));
	m_FileMenu.AppendMenu(MF_SEPARATOR);
	m_FileMenu.AppendMenu(MF_STRING,MP_OPEN, GetResString(IDS_DL_OPEN));
	m_FileMenu.AppendMenu(MF_STRING,MP_MOVIE, GetResString(IDS_MOVIE));
	m_FileMenu.AppendMenu(MF_STRING,MP_PREVIEW, GetResString(IDS_DL_PREVIEW));
	m_FileMenu.AppendMenu(MF_STRING,MP_METINFO, GetResString(IDS_DL_INFO));
	m_FileMenu.AppendMenu(MF_STRING,MP_VIEWFILECOMMENTS, GetResString(IDS_CMT_SHOWALL));
	m_FileMenu.AppendMenu(MF_STRING,MP_FAKECHECK1, (LPCTSTR)"Jigle FakeCheck"); // DeltaHF
	m_FileMenu.AppendMenu(MF_SEPARATOR);
	m_FileMenu.AppendMenu(MF_STRING,MP_LOADSOURCES, GetResString(IDS_DL_LOADSOURCES)); // Mr.Fry
	m_FileMenu.AppendMenu(MF_STRING,MP_SAVESOURCES, GetResString(IDS_DL_SAVESOURCES)); // Mr.Fry
	m_FileMenu.AppendMenu(MF_SEPARATOR);
	m_FileMenu.AppendMenu(MF_STRING,MP_DROPNONEEDEDSRCS, GetResString(IDS_DROPNONEEDEDSRCS)); // Tarod
	m_FileMenu.AppendMenu(MF_STRING,MP_ALL_A4AF_TO_HERE, GetResString(IDS_ALL_A4AF_TO_HERE)); // Tarod
	sprintf(buffer,"%s (%s)", GetResString(IDS_ALL_A4AF_TO_HERE), GetResString(IDS_PRIOAUTO));
	m_FileMenu.AppendMenu(MF_STRING,MP_ALL_A4AF_AUTO,buffer); // Tarod
	m_FileMenu.AppendMenu(MF_SEPARATOR);
	
	m_FileMenu.AppendMenu(MF_STRING,MP_GETED2KLINK, GetResString(IDS_DL_LINK1));
	m_FileMenu.AppendMenu(MF_STRING,MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2));
	
	if(m_hWnd)
		SetMenu(&m_FileMenu);

	EMULE_CATCH
}

void CDownloadListCtrl::OnSize(UINT nType, int cx, int cy)
{
	EMULE_TRY

	CListCtrl::OnSize(nType, cx, cy);
	Invalidate();

	EMULE_CATCH
}

// Original by deltHF and Rod modified by Tarod
// After that modded by Vorlost too 
void CDownloadListCtrl::RestartWaitingDownloads() 
{ 
	EMULE_TRY

	CTypedPtrList<CPtrList, CPartFile*> selectedList; 
	int count = GetItemCount(); 
	POSITION pos; 

	// Vorlost get initial sources from server if we dont have our MaxSourcesPerFile Limit reached 
	for(int i = 0; i != count; i++)
	{ 
		if(((CMuleCtrlItem*)GetItemData(i))->IsFile() 
			&& ((CMuleCtrlItem*)GetItemData(i))->GetFile()->GetSourceCount() <= theApp.glob_prefs->GetMaxSourcePerFile()) 
			selectedList.AddTail(((CMuleCtrlItem*)GetItemData(i))->GetFile());
   	} 

	count = selectedList.GetCount(); 

	for (pos = selectedList.GetHeadPosition() ; pos != 0 ; selectedList.GetNext(pos)) 
		selectedList.GetAt(pos)->GetSourcesAfterServerConnect(); 
    
	theApp.emuledlg->AddLogLine(true,GetResString(IDS_NEWSERVERCONNECT),theApp.glob_prefs->GetMaxSourcePerFile()); 

	selectedList.RemoveAll(); 

	EMULE_CATCH
}

void CDownloadListCtrl::DownloadAllA4AF(CPartFile* file)
{
	EMULE_TRY

	// Make a list of sources
	CArray<CUpDownClient*, CUpDownClient*> SourcesList;
	m_ItemMgr.LockRemove();
	MuleRange Range = m_ItemMgr.GetItemsByFile(file);
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		CMuleCtrlItem* pItem = (CMuleCtrlItem*)g->second;
		if(pItem->IsSource() && pItem->IsAskedForAnotherFile())
			SourcesList.Add(pItem->GetSource());
	}
	m_ItemMgr.UnlockRemove();
	// Actually handover
	for(int i = 0; i < SourcesList.GetCount(); i++)
		SourcesList[i]->HandoverToAnotherFile(file);

	EMULE_CATCH
}

void CDownloadListCtrl::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	EMULE_TRY

	RefreshInfo();
	CMuleListCtrl::OnKeyUp(nChar, nRepCnt, nFlags);

	EMULE_CATCH
}

void CDownloadListCtrl::RefreshInfo(void)
{
	EMULE_TRY

	POSITION pos = GetFirstSelectedItemPosition();
	if(pos == NULL)
	{			
		theApp.emuledlg->transferwnd.infolistctrl.SwitchToFile(NULL);
		theApp.emuledlg->transferwnd.UpdateInfoHeader();
	} else {
		CMuleCtrlItem* content = (CMuleCtrlItem*)GetItemData(GetSelectionMark());
		if(content->IsFile())
		{
			const CPartFile* file = content->GetFile();
			theApp.emuledlg->transferwnd.infolistctrl.SwitchToFile((CPartFile*)file);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
		} else {
			const CUpDownClient* client = content->GetSource();
			theApp.emuledlg->transferwnd.infolistctrl.SwitchToUser((CUpDownClient*)client);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
		}	
	}	

	EMULE_CATCH
}

CString CDownloadListCtrl::getTextList() 
{
	EMULE_TRY

	CString out="";
	CString temp,temp2;

	m_ItemMgr.LockRemove();
	MuleRange Range = m_ItemMgr.GetAllFiles();
	for(MuleIter g = Range.first; g != Range.second; g++)
	{
		CPartFile* file = (CPartFile*)g->second;
		theApp.emuledlg->AddLogLine(false,file->GetFileName());
		temp2=file->GetFileName(); if (temp2.GetLength()>50) temp2=temp2.Mid(0,50);
		temp.Format("\n%s\t [%.1f%%] %i/%i - %s",
			temp2,file->GetPercentCompleted(),file->GetTransferingSrcCount(),file->GetSourceCount(), file->GetPartfileStatus());
		out=out+temp;
    }
	theApp.emuledlg->AddLogLine(false,out);
	m_ItemMgr.UnlockRemove();

	return out;

	EMULE_CATCH

	return _T("");
}
